-------------------------------------------------------------------------------
-- Product Note PN122
-- VHDL module used in the 'RTDS MMC SFP' project
-- Copyright imperix ltd, 2024
-- dev@imperix.ch
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RTDS_firing_words is
    Generic(
        N : natural := 4
    );
    Port(
        clk : IN std_logic;

        pwm : IN std_logic_vector(2*N*6-1 downto 0);
        fast_discharge : IN std_logic_vector(N*6-1 downto 0);
        deactivate : IN std_logic_vector(N*6-1 downto 0);

        firing_words : OUT std_logic_vector(N*6*8-1 downto 0)
    );
end RTDS_firing_words;

architecture Behavioral of RTDS_firing_words is

begin

    generate_firing: for i in 0 to N*6-1 generate
        firing_words(i*8+7) <= '0';
        firing_words(i*8+6) <= fast_discharge(i);
        firing_words(i*8+5) <= deactivate(i);
        firing_words(i*8+4) <= '0';
        firing_words(i*8+3) <= '0';
        firing_words(i*8+2) <= '0';
        firing_words(i*8+1) <= pwm(2*i+1);
        firing_words(i*8+0) <= pwm(2*i);
    end generate;

end Behavioral;
