-------------------------------------------------------------------------------
-- Product Note PN122
-- VHDL module used in the 'RTDS MMC SFP' project
-- Copyright imperix ltd, 2024
-- dev@imperix.ch
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RTDS_serial_to_parallel_A is
    Port(
        clk : IN std_logic;

        IN_S_AXIS_tdata : IN std_logic_vector(31 downto 0);
        IN_S_AXIS_tvalid : IN std_logic;
        IN_S_AXIS_tlast : IN std_logic;

        v_branch_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        i_branch_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_valves_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_branch_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        i_branch_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_valves_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_branch_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        i_branch_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_valves_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_branch_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        i_branch_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_valves_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_branch_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        i_branch_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_valves_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_branch_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        i_branch_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        v_valves_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');

        passthru_1_13 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_2_13 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_3_13 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_4_13 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_5_13 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_6_13 : OUT std_logic_vector(31 downto 0) := (others => '0');

        passthru_1_46 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_2_46 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_3_46 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_4_46 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_5_46 : OUT std_logic_vector(31 downto 0) := (others => '0');
        passthru_6_46 : OUT std_logic_vector(31 downto 0) := (others => '0');

        out_valid : OUT std_logic := '0'
    );
end RTDS_serial_to_parallel_A;

architecture Behavioral of RTDS_serial_to_parallel_A is

    constant Ntot : natural := 30;

    type ff_chain_type is array (0 to Ntot-1) of std_logic_vector(31 downto 0);
    signal ff_chain : ff_chain_type := (others => (others => '0'));

    signal valid_last_ff : std_logic := '0';

    -- interface
    ATTRIBUTE X_INTERFACE_INFO : STRING;
    ATTRIBUTE X_INTERFACE_PARAMETER : STRING;

    ATTRIBUTE X_INTERFACE_INFO of v_branch_leg_1: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_000";
    ATTRIBUTE X_INTERFACE_INFO of i_branch_leg_1: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_001";
    ATTRIBUTE X_INTERFACE_INFO of v_valves_leg_1: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_002";
    ATTRIBUTE X_INTERFACE_INFO of v_branch_leg_2: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_003";
    ATTRIBUTE X_INTERFACE_INFO of i_branch_leg_2: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_004";
    ATTRIBUTE X_INTERFACE_INFO of v_valves_leg_2: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_005";
    ATTRIBUTE X_INTERFACE_INFO of v_branch_leg_3: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_006";
    ATTRIBUTE X_INTERFACE_INFO of i_branch_leg_3: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_007";
    ATTRIBUTE X_INTERFACE_INFO of v_valves_leg_3: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_008";
    ATTRIBUTE X_INTERFACE_INFO of v_branch_leg_4: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_009";
    ATTRIBUTE X_INTERFACE_INFO of i_branch_leg_4: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_010";
    ATTRIBUTE X_INTERFACE_INFO of v_valves_leg_4: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_011";
    ATTRIBUTE X_INTERFACE_INFO of v_branch_leg_5: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_012";
    ATTRIBUTE X_INTERFACE_INFO of i_branch_leg_5: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_013";
    ATTRIBUTE X_INTERFACE_INFO of v_valves_leg_5: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_014";
    ATTRIBUTE X_INTERFACE_INFO of v_branch_leg_6: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_015";
    ATTRIBUTE X_INTERFACE_INFO of i_branch_leg_6: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_016";
    ATTRIBUTE X_INTERFACE_INFO of v_valves_leg_6: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_017";
    ATTRIBUTE X_INTERFACE_INFO of passthru_1_13: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_018";
    ATTRIBUTE X_INTERFACE_INFO of passthru_2_13: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_019";
    ATTRIBUTE X_INTERFACE_INFO of passthru_3_13: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_020";
    ATTRIBUTE X_INTERFACE_INFO of passthru_4_13: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_021";
    ATTRIBUTE X_INTERFACE_INFO of passthru_5_13: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_022";
    ATTRIBUTE X_INTERFACE_INFO of passthru_6_13: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_023";
    ATTRIBUTE X_INTERFACE_INFO of passthru_1_46: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_024";
    ATTRIBUTE X_INTERFACE_INFO of passthru_2_46: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_025";
    ATTRIBUTE X_INTERFACE_INFO of passthru_3_46: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_026";
    ATTRIBUTE X_INTERFACE_INFO of passthru_4_46: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_027";
    ATTRIBUTE X_INTERFACE_INFO of passthru_5_46: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_028";
    ATTRIBUTE X_INTERFACE_INFO of passthru_6_46: SIGNAL is "imperix.ch:user:CUSTOM_BUS:1.0 signals_out port_029";

    ATTRIBUTE X_INTERFACE_INFO of clk: SIGNAL is "xilinx.com:signal:clock:1.0 clk CLK";
    ATTRIBUTE X_INTERFACE_PARAMETER of clk: SIGNAL is "FREQ_HZ 250000000, ASSOCIATED_BUSIF signals_out:IN_S_AXIS";

begin

    process(clk) begin
        if rising_edge(clk) then
            if IN_S_AXIS_tvalid = '1' then
                ff_chain(Ntot-1) <= IN_S_AXIS_tdata;

                for i in 0 to Ntot-2 loop
                    ff_chain(i) <= ff_chain(i+1);
                end loop;
            end if;

            valid_last_ff <= IN_S_AXIS_tvalid and IN_S_AXIS_tlast;
        end if;
    end process;

    process(clk) begin
        if rising_edge(clk) then
            if valid_last_ff = '1' then

                v_branch_leg_1 <= ff_chain(0);
                i_branch_leg_1 <= ff_chain(1);
                v_valves_leg_1 <= ff_chain(2);
                v_branch_leg_2 <= ff_chain(3);
                i_branch_leg_2 <= ff_chain(4);
                v_valves_leg_2 <= ff_chain(5);
                v_branch_leg_3 <= ff_chain(6);
                i_branch_leg_3 <= ff_chain(7);
                v_valves_leg_3 <= ff_chain(8);
                v_branch_leg_4 <= ff_chain(9);
                i_branch_leg_4 <= ff_chain(10);
                v_valves_leg_4 <= ff_chain(11);
                v_branch_leg_5 <= ff_chain(12);
                i_branch_leg_5 <= ff_chain(13);
                v_valves_leg_5 <= ff_chain(14);
                v_branch_leg_6 <= ff_chain(15);
                i_branch_leg_6 <= ff_chain(16);
                v_valves_leg_6 <= ff_chain(17);

                passthru_1_13  <= ff_chain(18);
                passthru_2_13  <= ff_chain(19);
                passthru_3_13  <= ff_chain(20);
                passthru_4_13  <= ff_chain(21);
                passthru_5_13  <= ff_chain(22);
                passthru_6_13  <= ff_chain(23);
                
                passthru_1_46  <= ff_chain(24);
                passthru_2_46  <= ff_chain(25);
                passthru_3_46  <= ff_chain(26);
                passthru_4_46  <= ff_chain(27);
                passthru_5_46  <= ff_chain(28);
                passthru_6_46  <= ff_chain(29);

                out_valid <= '1';
            else
                out_valid <= '0';
            end if;
        end if;
    end process;

end Behavioral;
